@@ -1,8 +1,7 @@
 package org.fdroid.k9.activity;
 
 import java.util.Date;
-import org.fdroid.k9.helper.MessageHelper;
-import org.fdroid.k9.mail.store.LocalStore.LocalMessage;
+import org.fdroid.k9.mail.Message;
 
 public class MessageInfoHolder {
     public String date;
@@ -16,11 +15,10 @@ public class MessageInfoHolder {
     public String uid;
     public boolean read;
     public boolean answered;
+    public boolean forwarded;
     public boolean flagged;
-    public boolean downloaded;
-    public boolean partially_downloaded;
     public boolean dirty;
-    public LocalMessage message;
+    public Message message;
     public FolderInfoHolder folder;
     public boolean selected;
     public String account;
@@ -44,11 +42,4 @@ public class MessageInfoHolder {
     public int hashCode() {
         return uid.hashCode();
     }
-
-    public String getDate(MessageHelper messageHelper) {
-        if (date == null) {
-            date = messageHelper.formatDate(message.getSentDate());
-        }
-        return date;
-    }
 }
