@@ -1,6 +1,7 @@
 package org.fdroid.k9.helper;
 
 import android.text.*;
+import android.text.Html.TagHandler;
 import android.util.Log;
 import org.fdroid.k9.K9;
 import org.xml.sax.XMLReader;
@@ -125,40 +126,29 @@ public class HtmlConverter {
 
     private static final int MAX_SMART_HTMLIFY_MESSAGE_LENGTH = 1024 * 256 ;
 
-    public static final String getHtmlHeader() {
-        return "<html><head/><body>";
-    }
-
-    public static final String getHtmlFooter() {
-        return "</body></html>";
-    }
-
     /**
      * Naively convert a text string into an HTML document.
      *
      * <p>
      * This method avoids using regular expressions on the entire message body to save memory.
      * </p>
+     * <p>
+     * No HTML headers or footers are added to the result.  Headers and footers
+     * are added at display time in
+     * {@link org.fdroid.k9.view#MessageWebView.setText(String) MessageWebView.setText()}
+     * </p>
      *
      * @param text
      *         Plain text string.
-     * @param useHtmlTag
-     *         If {@code true} this method adds headers and footers to create a proper HTML
-     *         document.
-     *
      * @return HTML string.
      */
-    private static String simpleTextToHtml(String text, boolean useHtmlTag) {
+    private static String simpleTextToHtml(String text) {
         // Encode HTML entities to make sure we don't display something evil.
         text = TextUtils.htmlEncode(text);
 
         StringReader reader = new StringReader(text);
         StringBuilder buff = new StringBuilder(text.length() + TEXT_TO_HTML_EXTRA_BUFFER_LENGTH);
 
-        if (useHtmlTag) {
-            buff.append(getHtmlHeader());
-        }
-
         buff.append(htmlifyMessageHeader());
 
         int c;
@@ -183,10 +173,6 @@ public class HtmlConverter {
 
         buff.append(htmlifyMessageFooter());
 
-        if (useHtmlTag) {
-            buff.append(getHtmlFooter());
-        }
-
         return buff.toString();
     }
 
@@ -200,26 +186,28 @@ public class HtmlConverter {
      * Convert a text string into an HTML document.
      *
      * <p>
-     * Attempts to do smart replacement for large documents to prevent OOM errors. This method
-     * optionally adds headers and footers to create a proper HTML document. To convert to a
-     * fragment, use {@link #textToHtmlFragment(String)}.
+     * Attempts to do smart replacement for large documents to prevent OOM
+     * errors.
+     * <p>
+     * No HTML headers or footers are added to the result.  Headers and footers
+     * are added at display time in
+     * {@link org.fdroid.k9.view#MessageWebView.setText(String) MessageWebView.setText()}
+     * </p>
+     * <p>
+     * To convert to a fragment, use {@link #textToHtmlFragment(String)} .
      * </p>
      *
      * @param text
      *         Plain text string.
-     * @param useHtmlTag
-     *         If {@code true} this method adds headers and footers to create a proper HTML
-     *         document.
-     *
      * @return HTML string.
      */
-    public static String textToHtml(String text, boolean useHtmlTag) {
+    public static String textToHtml(String text) {
         // Our HTMLification code is somewhat memory intensive
         // and was causing lots of OOM errors on the market
         // if the message is big and plain text, just do
         // a trivial htmlification
         if (text.length() > MAX_SMART_HTMLIFY_MESSAGE_LENGTH) {
-            return simpleTextToHtml(text, useHtmlTag);
+            return simpleTextToHtml(text);
         }
         StringReader reader = new StringReader(text);
         StringBuilder buff = new StringBuilder(text.length() + TEXT_TO_HTML_EXTRA_BUFFER_LENGTH);
@@ -247,7 +235,11 @@ public class HtmlConverter {
                     if (isStartOfLine) {
                         quotesThisLine++;
                     } else {
-                        buff.append("&gt;");
+                        // We use a token here which can't occur in htmlified text because &gt; is valid
+                        // within links (where > is not), and linkifying links will include it if we
+                        // do it here. We'll make another pass and change this back to &gt; after
+                        // the linkification is done.
+                        buff.append("<gt>");
                     }
                     break;
                 case '\r':
@@ -262,9 +254,6 @@ public class HtmlConverter {
                     if (isStartOfLine) {
                         // Not a quote character and not a space.  Content is starting now.
                         isStartOfLine = false;
-                        if (K9.DEBUG) {
-                            Log.d(K9.LOG_TAG, "currentQuoteDepth: " + quoteDepth + " quotesThisLine: " + quotesThisLine);
-                        }
                         // Add/remove blockquotes by comparing this line's quotes to the previous line's quotes.
                         if (quotesThisLine > quoteDepth) {
                             for (int i = quoteDepth; i < quotesThisLine; i++) {
@@ -310,20 +299,15 @@ public class HtmlConverter {
 
         StringBuffer sb = new StringBuffer(text.length() + TEXT_TO_HTML_EXTRA_BUFFER_LENGTH);
 
-        if (useHtmlTag) {
-            sb.append(getHtmlHeader());
-        }
-
         sb.append(htmlifyMessageHeader());
         linkifyText(text, sb);
         sb.append(htmlifyMessageFooter());
 
-        if (useHtmlTag) {
-            sb.append(getHtmlFooter());
-        }
-
         text = sb.toString();
 
+        // Above we replaced > with <gt>, now make it &gt;
+        text = text.replaceAll("<gt>", "&gt;");
+
         return text;
     }
 
@@ -333,6 +317,7 @@ public class HtmlConverter {
     protected static final String QUOTE_COLOR_LEVEL_3 = "#8ae234";
     protected static final String QUOTE_COLOR_LEVEL_4 = "#fcaf3e";
     protected static final String QUOTE_COLOR_LEVEL_5 = "#e9b96e";
+    private static final String K9MAIL_CSS_CLASS = "k9mail";
 
     /**
      * Return an HTML hex color string for a given quote level.
@@ -1246,16 +1231,34 @@ public class HtmlConverter {
     }
 
     private static String htmlifyMessageHeader() {
-        final String font = K9.messageViewFixedWidthFont()
-                            ? "monospace"
-                            : "sans-serif";
-        return "<pre style=\"white-space: pre-wrap; word-wrap:break-word; font-family: " + font + "; margin-top: 0px\">";
+        return "<pre class=\"" + K9MAIL_CSS_CLASS + "\">";
     }
 
     private static String htmlifyMessageFooter() {
         return "</pre>";
     }
 
+    /**
+     * Dynamically generate a CSS style for {@code <pre>} elements.
+     *
+     *  <p>
+     *  The style incorporates the user's current preference
+     *  setting for the font family used for plain text messages.
+     *  </p>
+     *
+     * @return
+     *      A {@code <style>} element that can be dynamically included in the HTML
+     *      {@code <head>} element when messages are displayed.
+     */
+    public static String cssStylePre() {
+        final String font = K9.messageViewFixedWidthFont()
+                ? "monospace"
+                : "sans-serif";
+        return "<style type=\"text/css\"> pre." + K9MAIL_CSS_CLASS +
+                " {white-space: pre-wrap; word-wrap:break-word; " +
+                "font-family: " + font + "; margin-top: 0px}</style>";
+    }
+
     /**
      * Convert a plain text string into an HTML fragment.
      * @param text Plain text.
@@ -1275,4 +1278,48 @@ public class HtmlConverter {
         // standard, but Gmail doesn't recognize it as an HTML entity. We unescape that here.
         return linkified.toString().replace("\n", "<br>\n").replace("&apos;", "&#39;");
     }
+
+    /**
+     * Convert HTML to a {@link Spanned} that can be used in a {@link android.widget.TextView}.
+     *
+     * @param html
+     *         The HTML fragment to be converted.
+     *
+     * @return A {@link Spanned} containing the text in {@code html} formatted using spans.
+     */
+    public static Spanned htmlToSpanned(String html) {
+        return Html.fromHtml(html, null, new ListTagHandler());
+    }
+
+    /**
+     * {@link TagHandler} that supports unordered lists.
+     *
+     * @see HtmlConverter#htmlToSpanned(String)
+     */
+    public static class ListTagHandler implements TagHandler {
+        @Override
+        public void handleTag(boolean opening, String tag, Editable output, XMLReader xmlReader) {
+            if (tag.equals("ul")) {
+                if (opening) {
+                    char lastChar = 0;
+                    if (output.length() > 0) {
+                        lastChar = output.charAt(output.length() - 1);
+                    }
+                    if (lastChar != '\n') {
+                        output.append("\n");
+                    }
+                } else {
+                    output.append("\n");
+                }
+            }
+
+            if (tag.equals("li")) {
+                if (opening) {
+                    output.append("\t•  ");
+                } else {
+                    output.append("\n");
+                }
+            }
+        }
+    }
 }
