@@ -1,7 +1,13 @@
 
 package org.fdroid.k9.helper;
 
+import android.annotation.SuppressLint;
+import android.app.Application;
+import android.content.Context;
 import android.database.Cursor;
+import android.net.ConnectivityManager;
+import android.net.NetworkInfo;
+import android.os.Build;
 import android.text.Editable;
 import android.util.Log;
 import android.widget.EditText;
@@ -14,7 +20,10 @@ import java.io.File;
 import java.io.FileInputStream;
 import java.io.FileOutputStream;
 import java.io.UnsupportedEncodingException;
+import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Date;
+import java.util.List;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
@@ -66,6 +75,15 @@ public class Utility {
         return false;
     }
 
+    public static boolean arrayContainsAny(Object[] a, Object... o) {
+        for (Object element : a) {
+            if (arrayContains(o, element)) {
+                return true;
+            }
+        }
+        return false;
+    }
+
     /**
      * Combines the given array of Objects into a single String using
      * each Object's toString() method and the separator character
@@ -209,21 +227,6 @@ public class Utility {
         }
     }
 
-    private static final long MILISECONDS_IN_18_HOURS = 18 * 60 * 60 * 1000;
-    /**
-     * Returns true if the specified date is within 18 hours of "now". Returns false otherwise.
-     * @param date
-     * @return
-     */
-    public static boolean isDateToday(Date date) {
-        Date now = new Date();
-        if (now.getTime() - MILISECONDS_IN_18_HOURS > date.getTime() || now.getTime() + MILISECONDS_IN_18_HOURS < date.getTime()) {
-            return false;
-        } else {
-            return true;
-        }
-    }
-
     /*
      * TODO disabled this method globally. It is used in all the settings screens but I just
      * noticed that an unrelated icon was dimmed. Android must share drawables internally.
@@ -640,4 +643,76 @@ public class Utility {
     public static String sanitizeFilename(String filename) {
         return filename.replaceAll(INVALID_CHARACTERS, REPLACEMENT_CHARACTER);
     }
+
+    /**
+     * Check to see if we have network connectivity.
+     * @param app Current application (Hint: see if your base class has a getApplication() method.)
+     * @return true if we have connectivity, false otherwise.
+     */
+    public static boolean hasConnectivity(final Application app) {
+        final ConnectivityManager connectivityManager =
+            (ConnectivityManager) app.getSystemService(Context.CONNECTIVITY_SERVICE);
+        if (connectivityManager == null) {
+            return false;
+        }
+        final NetworkInfo netInfo = connectivityManager.getActiveNetworkInfo();
+        if (netInfo != null && netInfo.getState() == NetworkInfo.State.CONNECTED) {
+            return true;
+        } else {
+            return false;
+        }
+    }
+
+    private static final Pattern MESSAGE_ID = Pattern.compile("<" +
+            "(?:" +
+                "[a-zA-Z0-9!#$%&'*+\\-/=?^_`{|}~]+" +
+                "(?:\\.[a-zA-Z0-9!#$%&'*+\\-/=?^_`{|}~]+)*" +
+                "|" +
+                "\"(?:[^\\\\\"]|\\\\.)*\"" +
+            ")" +
+            "@" +
+            "(?:" +
+                "[a-zA-Z0-9!#$%&'*+\\-/=?^_`{|}~]+" +
+                "(?:\\.[a-zA-Z0-9!#$%&'*+\\-/=?^_`{|}~]+)*" +
+                "|" +
+                "\\[(?:[^\\\\\\]]|\\\\.)*\\]" +
+            ")" +
+            ">");
+
+    public static List<String> extractMessageIds(final String text) {
+        List<String> messageIds = new ArrayList<String>();
+        Matcher matcher = MESSAGE_ID.matcher(text);
+
+        int start = 0;
+        while (matcher.find(start)) {
+            String messageId = text.substring(matcher.start(), matcher.end());
+            messageIds.add(messageId);
+            start = matcher.end();
+        }
+
+        return messageIds;
+    }
+
+    public static String extractMessageId(final String text) {
+        Matcher matcher = MESSAGE_ID.matcher(text);
+
+        if (matcher.find()) {
+            return text.substring(matcher.start(), matcher.end());
+        }
+
+        return null;
+    }
+
+    @SuppressLint("NewApi")
+    public static String[] copyOf(String[] original, int newLength) {
+        if (Build.VERSION.SDK_INT >= 9) {
+            return Arrays.copyOf(original, newLength);
+        }
+
+        String[] newArray = new String[newLength];
+        int copyLength = (original.length >= newLength) ? newLength : original.length;
+        System.arraycopy(original, 0, newArray, 0, copyLength);
+
+        return newArray;
+    }
 }
