@@ -58,8 +58,6 @@ public class Pop3Store extends Store {
     private static final String USER_CAPABILITY = "USER";
     private static final String TOP_CAPABILITY = "TOP";
 
-    private static final Flag[] PERMANENT_FLAGS = { Flag.DELETED };
-
     /**
      * Decodes a Pop3Store URI.
      *
@@ -289,6 +287,11 @@ public class Pop3Store extends Store {
         folder.close();
     }
 
+    @Override
+    public boolean isSeenFlagSupported() {
+        return false;
+    }
+
     class Pop3Folder extends Folder {
         private Socket mSocket;
         private InputStream mIn;
@@ -564,8 +567,13 @@ public class Pop3Store extends Store {
                     Pop3Message message = mMsgNumToMsgMap.get(msgNum);
                     if (message == null) {
                         String response = executeSimpleCommand(UIDL_COMMAND + " " + msgNum);
-                        int uidIndex = response.lastIndexOf(' ');
-                        String msgUid = response.substring(uidIndex + 1);
+                        // response = "+OK msgNum msgUid"
+                        String[] uidParts = response.split(" +");
+                        if (uidParts.length < 3 || !"+OK".equals(uidParts[0])) {
+                            Log.e(K9.LOG_TAG, "ERR response: " + response);
+                            return;
+                        }
+                        String msgUid = uidParts[2];
                         message = new Pop3Message(msgUid, this);
                         indexMessage(msgNum, message);
                     }
@@ -639,7 +647,7 @@ public class Pop3Store extends Store {
                 if (response.equals(".")) {
                     break;
                 }
-                String[] uidParts = response.split(" ");
+                String[] uidParts = response.split(" +");
 
                 // Ignore messages without a unique-id
                 if (uidParts.length >= 2) {
@@ -884,11 +892,6 @@ public class Pop3Store extends Store {
             }
         }
 
-        @Override
-        public Flag[] getPermanentFlags() {
-            return PERMANENT_FLAGS;
-        }
-
         @Override
         public Map<String, String> appendMessages(Message[] messages) throws MessagingException {
             return null;
@@ -1159,6 +1162,8 @@ public class Pop3Store extends Store {
      * Exception that is thrown if the server returns an error response.
      */
     static class Pop3ErrorResponse extends MessagingException {
+        private static final long serialVersionUID = 3672087845857867174L;
+
         public Pop3ErrorResponse(String message) {
             super(message, true);
         }
