@@ -77,13 +77,13 @@ public class SmtpTransport extends Transport {
         String scheme = smtpUri.getScheme();
         if (scheme.equals("smtp")) {
             connectionSecurity = ConnectionSecurity.NONE;
-            port = 25;
+            port = 587;
         } else if (scheme.equals("smtp+tls")) {
             connectionSecurity = ConnectionSecurity.STARTTLS_OPTIONAL;
-            port = 25;
+            port = 587;
         } else if (scheme.equals("smtp+tls+")) {
             connectionSecurity = ConnectionSecurity.STARTTLS_REQUIRED;
-            port = 25;
+            port = 587;
         } else if (scheme.equals("smtp+ssl+")) {
             connectionSecurity = ConnectionSecurity.SSL_TLS_REQUIRED;
             port = 465;
@@ -182,6 +182,7 @@ public class SmtpTransport extends Transport {
     }
 
 
+    Account mAccount;
     String mHost;
     int mPort;
     String mUsername;
@@ -194,14 +195,15 @@ public class SmtpTransport extends Transport {
     private boolean m8bitEncodingAllowed;
     private int mLargestAcceptableMessage;
 
-    public SmtpTransport(String uri) throws MessagingException {
+    public SmtpTransport(Account account) throws MessagingException {
         ServerSettings settings;
         try {
-            settings = decodeUri(uri);
+            settings = decodeUri(account.getTransportUri());
         } catch (IllegalArgumentException e) {
             throw new MessagingException("Error while decoding transport URI", e);
         }
 
+        mAccount = account;
         mHost = settings.host;
         mPort = settings.port;
 
