@@ -22,7 +22,6 @@ import org.fdroid.k9.Account;
 import org.fdroid.k9.Identity;
 import org.fdroid.k9.K9;
 import org.fdroid.k9.Preferences;
-import org.fdroid.k9.helper.DateFormatter;
 import org.fdroid.k9.helper.Utility;
 import org.fdroid.k9.mail.ConnectionSecurity;
 import org.fdroid.k9.mail.ServerSettings;
@@ -293,7 +292,6 @@ public class SettingsImporter {
             }
 
             preferences.loadAccounts();
-            DateFormatter.clearChosenFormat();
             K9.loadPrefs(preferences);
             K9.setServicesEnabled(context);
 
@@ -598,6 +596,10 @@ public class SettingsImporter {
 
     private static boolean isAccountNameUsed(String name, Account[] accounts) {
         for (Account account : accounts) {
+            if (account == null) {
+                continue;
+            }
+
             if (account.getDescription().equals(name)) {
                 return true;
             }
