@@ -13,6 +13,7 @@ import android.database.CursorWindow;
 import android.database.DataSetObserver;
 import android.database.MatrixCursor;
 import android.net.Uri;
+import android.os.Binder;
 import android.os.Bundle;
 import android.provider.BaseColumns;
 import android.util.Log;
@@ -21,21 +22,22 @@ import org.fdroid.k9.Account;
 import org.fdroid.k9.AccountStats;
 import org.fdroid.k9.K9;
 import org.fdroid.k9.Preferences;
-import org.fdroid.k9.SearchAccount;
 import org.fdroid.k9.activity.FolderInfoHolder;
 import org.fdroid.k9.activity.MessageInfoHolder;
-import org.fdroid.k9.activity.MessageList;
 import org.fdroid.k9.controller.MessagingController;
 import org.fdroid.k9.controller.MessagingListener;
 import org.fdroid.k9.helper.MessageHelper;
+import org.fdroid.k9.mail.Flag;
 import org.fdroid.k9.mail.Folder;
 import org.fdroid.k9.mail.Message;
 import org.fdroid.k9.mail.MessagingException;
 import org.fdroid.k9.mail.store.LocalStore;
+import org.fdroid.k9.search.SearchAccount;
 
 import java.lang.ref.WeakReference;
 import java.util.ArrayList;
 import java.util.Collections;
+import java.util.Comparator;
 import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.concurrent.BlockingQueue;
@@ -81,7 +83,31 @@ public class MessageProvider extends ContentProvider {
          */
         String UNREAD = "unread";
 
+        /**
+         * <P>Type: TEXT</P>
+         */
         String ACCOUNT = "account";
+
+        /**
+         * <P>Type: INTEGER</P>
+         */
+        String ACCOUNT_NUMBER = "accountNumber";
+
+        /**
+         * <P>Type: BOOLEAN</P>
+         */
+        String HAS_ATTACHMENTS = "hasAttachments";
+
+        /**
+         * <P>Type: BOOLEAN</P>
+         */
+        String HAS_STAR = "hasStar";
+
+        /**
+         * <P>Type: INTEGER</P>
+         */
+        String ACCOUNT_COLOR = "accountColor";
+
         String URI = "uri";
         String DELETE_URI = "delUri";
 
@@ -197,6 +223,34 @@ public class MessageProvider extends ContentProvider {
         }
     }
 
+    public static class AccountColorExtractor implements FieldExtractor<MessageInfoHolder, Integer> {
+        @Override
+        public Integer getField(final MessageInfoHolder source) {
+            return source.message.getFolder().getAccount().getChipColor();
+        }
+    }
+
+    public static class AccountNumberExtractor implements FieldExtractor<MessageInfoHolder, Integer> {
+        @Override
+        public Integer getField(final MessageInfoHolder source) {
+            return source.message.getFolder().getAccount().getAccountNumber();
+        }
+    }
+
+    public static class HasAttachmentsExtractor implements FieldExtractor<MessageInfoHolder, Boolean> {
+        @Override
+        public Boolean getField(final MessageInfoHolder source) {
+            return source.message.hasAttachments();
+        }
+    }
+
+    public static class HasStarExtractor implements FieldExtractor<MessageInfoHolder, Boolean> {
+        @Override
+        public Boolean getField(final MessageInfoHolder source) {
+            return source.message.isSet(Flag.FLAGGED);
+        }
+    }
+
     public static class UnreadExtractor implements FieldExtractor<MessageInfoHolder, Boolean> {
         @Override
         public Boolean getField(final MessageInfoHolder source) {
@@ -205,7 +259,7 @@ public class MessageProvider extends ContentProvider {
     }
 
     /**
-     * @deprecated having an incremential value has no real interest,
+     * @deprecated having an incremental value has no real interest,
      *             implemented for compatibility only
      */
     @Deprecated
@@ -248,14 +302,13 @@ public class MessageProvider extends ContentProvider {
             final SearchAccount integratedInboxAccount = SearchAccount.createUnifiedInboxAccount(getContext());
             final MessagingController msgController = MessagingController.getInstance(K9.app);
 
-            msgController.searchLocalMessages(integratedInboxAccount, null,
+            msgController.searchLocalMessages(integratedInboxAccount.getRelatedSearch(),
                                               new MesssageInfoHolderRetrieverListener(queue));
 
             final List<MessageInfoHolder> holders = queue.take();
 
             // TODO add sort order parameter
-            Collections.sort(holders, new MessageList.ReverseComparator<MessageInfoHolder>(
-                                 new MessageList.DateComparator()));
+            Collections.sort(holders, new ReverseDateComparator());
 
             final String[] projectionToUse;
             if (projection == null) {
@@ -311,10 +364,18 @@ public class MessageProvider extends ContentProvider {
                     extractors.put(field, new UriExtractor());
                 } else if (MessageColumns.DELETE_URI.equals(field)) {
                     extractors.put(field, new DeleteUriExtractor());
-                } else if (MessageColumns.ACCOUNT.equals(field)) {
-                    extractors.put(field, new AccountExtractor());
                 } else if (MessageColumns.UNREAD.equals(field)) {
                     extractors.put(field, new UnreadExtractor());
+                } else if (MessageColumns.ACCOUNT.equals(field)) {
+                    extractors.put(field, new AccountExtractor());
+                } else if (MessageColumns.ACCOUNT_COLOR.equals(field)) {
+                    extractors.put(field, new AccountColorExtractor());
+                } else if (MessageColumns.ACCOUNT_NUMBER.equals(field)) {
+                    extractors.put(field, new AccountNumberExtractor());
+                } else if (MessageColumns.HAS_ATTACHMENTS.equals(field)) {
+                    extractors.put(field, new HasAttachmentsExtractor());
+                } else if (MessageColumns.HAS_STAR.equals(field)) {
+                    extractors.put(field, new HasStarExtractor());
                 } else if (MessageColumns.INCREMENT.equals(field)) {
                     extractors.put(field, new IncrementExtractor());
                 }
@@ -328,6 +389,10 @@ public class MessageProvider extends ContentProvider {
      * Retrieve the account list.
      */
     protected class AccountsQueryHandler implements QueryHandler {
+        private static final String FIELD_ACCOUNT_NUMBER = "accountNumber";
+        private static final String FIELD_ACCOUNT_NAME = "accountName";
+        private static final String FIELD_ACCOUNT_UUID = "accountUuid";
+        private static final String FIELD_ACCOUNT_COLOR = "accountColor";
 
         @Override
         public String getPath() {
@@ -337,18 +402,39 @@ public class MessageProvider extends ContentProvider {
         @Override
         public Cursor query(final Uri uri, String[] projection, String selection,
                             String[] selectionArgs, String sortOrder) throws Exception {
-            return getAllAccounts();
+            return getAllAccounts(projection);
         }
 
-        public Cursor getAllAccounts() {
-            String[] projection = new String[] { "accountNumber", "accountName" };
+        public Cursor getAllAccounts(String[] projection) {
+            // Default projection
+            if(projection == null) {
+                projection = new String[] { FIELD_ACCOUNT_NUMBER, FIELD_ACCOUNT_NAME };
+            }
+
 
             MatrixCursor ret = new MatrixCursor(projection);
 
             for (Account account : Preferences.getPreferences(getContext()).getAccounts()) {
-                Object[] values = new Object[2];
-                values[0] = account.getAccountNumber();
-                values[1] = account.getDescription();
+                Object[] values = new Object[projection.length];
+
+                // Build account row
+                int fieldIndex = 0;
+                for(String field : projection) {
+
+                    if(FIELD_ACCOUNT_NUMBER.equals(field)) {
+                        values[fieldIndex] = account.getAccountNumber();
+                    } else if(FIELD_ACCOUNT_NAME.equals(field)) {
+                        values[fieldIndex] = account.getDescription();
+                    } else if(FIELD_ACCOUNT_UUID.equals(field)) {
+                        values[fieldIndex] = account.getUuid();
+                    } else if(FIELD_ACCOUNT_COLOR.equals(field)) {
+                        values[fieldIndex] = account.getChipColor();
+                    } else {
+                        values[fieldIndex] = null;
+                    }
+                    ++fieldIndex;
+                }
+
                 ret.addRow(values);
             }
 
@@ -375,10 +461,22 @@ public class MessageProvider extends ContentProvider {
             int accountId = -1;
             segments = uri.getPathSegments();
             accountId = Integer.parseInt(segments.get(1));
-            return getAccountUnread(accountId);
+
+            /*
+             * This method below calls Account.getStats() which uses EmailProvider to do its work.
+             * For this to work we need to clear the calling identity. Otherwise accessing
+             * EmailProvider will fail because it's not exported so third-party apps can't access it
+             * directly.
+             */
+            long identityToken = Binder.clearCallingIdentity();
+            try {
+                return getAccountUnread(accountId);
+            } finally {
+                Binder.restoreCallingIdentity(identityToken);
+            }
         }
 
-        public Cursor getAccountUnread(int accountNumber) {
+        private Cursor getAccountUnread(int accountNumber) {
             String[] projection = new String[] { "accountName", "unread" };
 
             MatrixCursor ret = new MatrixCursor(projection);
@@ -626,6 +724,7 @@ public class MessageProvider extends ContentProvider {
             return mCursor.isFirst();
         }
 
+        @Override
         public boolean isLast() {
             checkClosed();
             return mCursor.isLast();
@@ -685,6 +784,7 @@ public class MessageProvider extends ContentProvider {
             mCursor.registerDataSetObserver(observer);
         }
 
+        @SuppressWarnings("deprecation")
         @Override
         public boolean requery() {
             checkClosed();
@@ -740,12 +840,19 @@ public class MessageProvider extends ContentProvider {
                             String sortOrder) throws Exception {
             mSemaphore.acquire();
 
-            final Cursor cursor;
-            cursor = mDelegate.query(uri, projection, selection, selectionArgs, sortOrder);
+            Cursor cursor = null;
+            try {
+                cursor = mDelegate.query(uri, projection, selection, selectionArgs, sortOrder);
+            } finally {
+                if (cursor == null) {
+                    mSemaphore.release();
+                }
+            }
 
             /* Android content resolvers can only process CrossProcessCursor instances */
             if (!(cursor instanceof CrossProcessCursor)) {
                 Log.w(K9.LOG_TAG, "Unsupported cursor, returning null: " + cursor);
+                mSemaphore.release();
                 return null;
             }
 
@@ -879,7 +986,7 @@ public class MessageProvider extends ContentProvider {
 
                 MessagingController.getInstance(application).addListener(new MessagingListener() {
                     @Override
-                    public void searchStats(final AccountStats stats) {
+                    public void folderStatusChanged(Account account, String folderName, int unreadMessageCount) {
                         application.getContentResolver().notifyChange(CONTENT_URI, null);
                     }
                 });
@@ -938,7 +1045,8 @@ public class MessageProvider extends ContentProvider {
 
         // launch command to delete the message
         if ((myAccount != null) && (msg != null)) {
-            MessagingController.getInstance(K9.app).deleteMessages(new Message[] { msg }, null);
+            MessagingController controller = MessagingController.getInstance(K9.app);
+            controller.deleteMessages(Collections.singletonList(msg), null);
         }
 
         // FIXME return the actual number of deleted messages
@@ -1036,4 +1144,16 @@ public class MessageProvider extends ContentProvider {
         mUriMatcher.addURI(AUTHORITY, handler.getPath(), code);
     }
 
+    public static class ReverseDateComparator implements Comparator<MessageInfoHolder> {
+        @Override
+        public int compare(MessageInfoHolder object2, MessageInfoHolder object1) {
+            if (object1.compareDate == null) {
+                return (object2.compareDate == null ? 0 : 1);
+            } else if (object2.compareDate == null) {
+                return -1;
+            } else {
+                return object1.compareDate.compareTo(object2.compareDate);
+            }
+        }
+    }
 }
