@@ -11,8 +11,10 @@ import android.os.AsyncTask;
 import android.os.Build;
 import android.os.Parcel;
 import android.os.Parcelable;
+import android.support.v4.app.Fragment;
 import android.util.AttributeSet;
 import android.util.Log;
+import android.util.TypedValue;
 import android.view.ContextMenu;
 import android.view.ContextMenu.ContextMenuInfo;
 import android.view.KeyEvent;
@@ -31,11 +33,11 @@ import android.widget.Toast;
 import org.fdroid.k9.Account;
 import org.fdroid.k9.K9;
 import org.fdroid.k9.R;
-import org.fdroid.k9.activity.K9Activity;
 import org.fdroid.k9.controller.MessagingController;
 import org.fdroid.k9.controller.MessagingListener;
 import org.fdroid.k9.crypto.CryptoProvider;
 import org.fdroid.k9.crypto.PgpData;
+import org.fdroid.k9.fragment.MessageViewFragment;
 import org.fdroid.k9.helper.ClipboardManager;
 import org.fdroid.k9.helper.Contacts;
 import org.fdroid.k9.helper.HtmlConverter;
@@ -108,7 +110,8 @@ public class SingleMessageView extends LinearLayout implements OnClickListener,
     private String mText;
 
 
-    public void initialize(Activity activity) {
+    public void initialize(Fragment fragment) {
+        Activity activity = fragment.getActivity();
         mMessageContentView = (MessageWebView) findViewById(R.id.message_content);
         mAccessibleMessageContentView = (AccessibleWebView) findViewById(R.id.accessible_message_content);
         mMessageContentView.configure();
@@ -128,7 +131,7 @@ public class SingleMessageView extends LinearLayout implements OnClickListener,
         mShowHiddenAttachments = (Button) findViewById(R.id.show_hidden_attachments);
         mShowHiddenAttachments.setVisibility(View.GONE);
         mCryptoView = (MessageCryptoView) findViewById(R.id.layout_decrypt);
-        mCryptoView.setActivity(activity);
+        mCryptoView.setFragment(fragment);
         mCryptoView.setupChildViews();
         mShowPicturesAction = findViewById(R.id.show_pictures);
         mShowMessageAction = findViewById(R.id.show_message);
@@ -139,7 +142,7 @@ public class SingleMessageView extends LinearLayout implements OnClickListener,
 
         mContacts = Contacts.getInstance(activity);
 
-        mInflater = activity.getLayoutInflater();
+        mInflater = ((MessageViewFragment) fragment).getFragmentLayoutInflater();
         mDownloadRemainder = (Button) findViewById(R.id.download_remainder);
         mDownloadRemainder.setVisibility(View.GONE);
         mAttachmentsContainer.setVisibility(View.GONE);
@@ -157,7 +160,11 @@ public class SingleMessageView extends LinearLayout implements OnClickListener,
             mHeaderPlaceHolder.removeView(mHeaderContainer);
             // the HTC version of WebView tries to force the background of the
             // titlebar, which is really unfair.
-            mHeaderContainer.setBackgroundColor(((K9Activity)activity).getThemeBackgroundColor());
+            TypedValue outValue = new TypedValue();
+            getContext().getTheme().resolveAttribute(R.attr.messageViewHeaderBackgroundColor, outValue, true);
+            mHeaderContainer.setBackgroundColor(outValue.data);
+            // also set background of the whole view (including the attachments view)
+            setBackgroundColor(outValue.data);
 
             mTitleBarHeaderContainer = new LinearLayout(activity);
             mMessageContentView.setEmbeddedTitleBarCompat(mTitleBarHeaderContainer);
@@ -178,6 +185,11 @@ public class SingleMessageView extends LinearLayout implements OnClickListener,
 
         WebView webview = (WebView) v;
         WebView.HitTestResult result = webview.getHitTestResult();
+
+        if (result == null) {
+            return;
+        }
+
         int type = result.getType();
         Context context = getContext();
 
@@ -393,7 +405,7 @@ public class SingleMessageView extends LinearLayout implements OnClickListener,
                 // Allow network access first...
                 setLoadPictures(true);
                 // ...then re-populate the WebView with the message text
-                loadBodyFromText(mText, "text/html");
+                loadBodyFromText(mText);
                 break;
             }
         }
@@ -495,6 +507,15 @@ public class SingleMessageView extends LinearLayout implements OnClickListener,
         mShowAttachmentsAction.setVisibility(show ? View.VISIBLE : View.GONE);
     }
 
+    /**
+     * Fetch the message header view.  This is not the same as the message headers; this is the View shown at the top
+     * of messages.
+     * @return MessageHeader View.
+     */
+    public MessageHeader getMessageHeaderView() {
+        return mHeaderContainer;
+    }
+
     public void setHeaders(final Message message, Account account) {
         try {
             mHeaderContainer.populate(message, account);
@@ -535,7 +556,7 @@ public class SingleMessageView extends LinearLayout implements OnClickListener,
         if (pgpData != null) {
             text = pgpData.getDecryptedData();
             if (text != null) {
-                text = HtmlConverter.textToHtml(text, true);
+                text = HtmlConverter.textToHtml(text);
             }
         }
 
@@ -595,7 +616,7 @@ public class SingleMessageView extends LinearLayout implements OnClickListener,
         }
 
         if (text != null) {
-            loadBodyFromText(text, "text/html");
+            loadBodyFromText(text);
             updateCryptoLayout(account.getCryptoProvider(), pgpData, message);
         } else {
             showStatusMessage(getContext().getString(R.string.webview_empty_message));
@@ -606,16 +627,15 @@ public class SingleMessageView extends LinearLayout implements OnClickListener,
         String text = "<html><body><div style=\"text-align:center; color: grey;\">" +
                 status +
                 "</div></body></html>";
-        loadBodyFromText(text, "text/html");
+        loadBodyFromText(text);
         mCryptoView.hide();
     }
 
-    private void loadBodyFromText(String emailText, String contentType) {
+    private void loadBodyFromText(String emailText) {
         if (mScreenReaderEnabled) {
-            mAccessibleMessageContentView.loadDataWithBaseURL("http://", emailText, contentType, "utf-8", null);
+            mAccessibleMessageContentView.setText(emailText);
         } else {
-            mMessageContentView.setText(emailText, contentType);
-            mMessageContentView.scrollTo(0, 0);
+            mMessageContentView.setText(emailText);
         }
 
     }
@@ -719,7 +739,7 @@ public class SingleMessageView extends LinearLayout implements OnClickListener,
          * its size because the button to download the complete message was previously shown and
          * is now hidden.
          */
-        loadBodyFromText("", "text/plain");
+        loadBodyFromText("");
     }
 
     public void resetHeaderView() {
@@ -789,7 +809,6 @@ public class SingleMessageView extends LinearLayout implements OnClickListener,
         boolean hiddenAttachmentsVisible;
         boolean showPictures;
 
-        @SuppressWarnings("hiding")
         public static final Parcelable.Creator<SavedState> CREATOR =
                 new Parcelable.Creator<SavedState>() {
             @Override
